;~ This file is part of ClipKeeper.

;~     ClipKeeper is free software: you can redistribute it and/or modify
;~     it under the terms of the GNU General Public License as published by
;~     the Free Software Foundation, either version 3 of the License, or
;~     (at your option) any later version.

;~     ClipKeeper is distributed in the hope that it will be useful,
;~     but WITHOUT ANY WARRANTY; without even the implied warranty of
;~     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;~     GNU General Public License for more details.

;~     You should have received a copy of the GNU General Public License
;~     along with ClipKeeper.  If not, see <http://www.gnu.org/licenses/>.
#include-once
#include <GuiImageList.au3>
#include <WinAPI.au3>

Global 			$AddIcon_ImageList[1][5]
Global Const	$AddIcon_ID			= 0
Global Const	$AddIcon_DOSSIER	= 1
Global Const	$AddIcon_EXE		= 2
Global Const	$AddIcon_INCONNU	= 3
Global Const	$AddIcon_TAB		= 4


; #FUNCTION# ====================================================================================================
; Name...........:	_AddIcon_ExplorerLikeByFileName
; Description....:	Ajoute la mme image du fichier $FileName que l'explorer windows afficherai
; Syntax.........:	_AddIcon_ExplorerLikeByFileName($hImages, $FileName)
; Parameters.....:	$hImages		- Variable retourne par _GUIImageList_Create()
;					$FileName		- Nom du fichier
;
; Return values..:	L'indice de l'image contenu dans l $hImages
; Author.........:	TommyDDR (http://tombarthelemy.fr)
; Modified.......:
; Remarks........:
; Related........:
; Link...........:
; Example........:
; ===============================================================================================================
Func _AddIcon_ExplorerLikeByFileName($hImages, $FileName)
	Local $Indice = __AddIcon_VerifImages($hImages)
	If($Indice < 0) Then
		$Indice = _AddIcon_RegisterImageList($hImages)
	EndIf
	Local $Image = -1
	If(StringInStr(FileGetAttrib($FileName), "D")) Then
		$Image = $AddIcon_ImageList[$Indice][1]
	Else
		Local $File
		Local $Index
		Local $Ext = __GetExtension($FileName)
		If($Ext = "ico") Then
			$File = $FileName
			$Index = 0
		Else
			Local $Registre = __GetNomIconByExt($Ext)
			If(Not(FileExists($Registre[0]))) Then
				$Registre[0] = @SystemDir & "\" & $Registre[0]
			EndIf
			If(Not(FileExists($Registre[0]))) Then
				$Registre[0] = _WinAPI_FindExecutable($FileName)
				$Registre[1] = 0
			EndIf
			$File = $Registre[0]
			$Index = $Registre[1]
			If($File = "") Then
				If($Ext = "exe") Then
					$Image = $AddIcon_ImageList[$Indice][2]
				Else
					$Image = $AddIcon_ImageList[$Indice][3]
				EndIf
			EndIf
		EndIf
		If($Image = -1) Then
			$Image = _AddIcon_ByFileName($hImages, $File, $Index)
		EndIf
	EndIf
	Return $Image
EndFunc

; #FUNCTION# ====================================================================================================
; Name...........:	_AddIcon_ByFileName
; Description....:	Ajoute l'image contenue dans le fichier $FileName.
; Syntax.........:	_AddIcon_ByFileName($hImages, $FileName, $Index)
; Parameters.....:	$hImages		- Variable retourne par _GUIImageList_Create()
;					$FileName		- Nom du fichier
;					$Index			- Index de la ressource du fichier
;
; Return values..:	L'indice de l'image contenu dans l $hImages
; Author.........:	TommyDDR (http://tombarthelemy.fr)
; Modified.......:
; Remarks........:
; Related........:
; Link...........:
; Example........:
; ===============================================================================================================
Func _AddIcon_ByFileName($hImages, $FileName, $Index = 0)
	Local $Indice = __AddIcon_VerifImages($hImages)
	If($Indice < 0) Then
		$Indice = _AddIcon_RegisterImageList($hImages)
	EndIf
	Local $Image = -1
	Local $Exist = __AddIcon_Exist($Indice, $FileName, $Index)
	If($Exist = -1) Then
		$Image = __AddIcon_Add($hImages, $Indice, $FileName, $Index)
	Else
		$Image = $Exist
	EndIf
	Return $Image
EndFunc

Func _AddIcon_RegisterImageList($hImages)
	Local $Indice = __AddIcon_VerifImages($hImages)
	If($Indice < 0) Then
		Local $TabTemp[1][3]
		$Indice = UBound($AddIcon_ImageList, 1)-1
		$AddIcon_ImageList[$Indice][$AddIcon_ID]		= $hImages
		$AddIcon_ImageList[$Indice][$AddIcon_DOSSIER]	= _GUIImageList_AddIcon($hImages, @SystemDir & "\shell32.dll", -4)
		$AddIcon_ImageList[$Indice][$AddIcon_EXE]		= _GUIImageList_AddIcon($hImages, @SystemDir & "\shell32.dll", -3)
		$AddIcon_ImageList[$Indice][$AddIcon_INCONNU]	= _GUIImageList_AddIcon($hImages, @SystemDir & "\shell32.dll", 0)
		$AddIcon_ImageList[$Indice][$AddIcon_TAB]		= $TabTemp
		ReDim $AddIcon_ImageList[UBound($AddIcon_ImageList, 1)+1][UBound($AddIcon_ImageList, 2)]
	EndIf
	Return $Indice
EndFunc

Func __GetExtension($Path)
	Local $Ext = StringSplit($Path, ".", 3)
	Return $Ext[UBound($Ext)-1]
EndFunc

Func __GetNomIconByExt($Ext)
	Local $Retour[2] = [-1, -1]
	Local $Nom = RegRead("HKEY_CLASSES_ROOT\." & $Ext, "")
	Local $NomIcon = RegRead("HKEY_CLASSES_ROOT\" & $Nom & "\DefaultIcon", "")
	If(Not(@error)) Then
		$NomIcon = StringSplit($NomIcon, ",", 3)
		$Retour[0] = _WinAPI_ExpandEnvironmentStrings($NomIcon[0])
		If(UBound($NomIcon) > 1) Then
			$Retour[1] = $NomIcon[1]
		Else
			$Retour[1] = 0
		EndIf
	Else
		$NomIcon = RegRead("HKEY_CLASSES_ROOT\" & $Nom & "\shell\open\command", "")
		If(Not(@error)) Then
			$NomIcon = StringSplit($NomIcon, """", 3)
			If(UBound($NomIcon) > 1) Then
				$Retour[0] = _WinAPI_ExpandEnvironmentStrings($NomIcon[1])
			Else
				$Retour[0] = _WinAPI_ExpandEnvironmentStrings($NomIcon[0])
			EndIf
			$Retour[1] = 0
		EndIf
	EndIf
	Return $Retour
EndFunc

Func __AddIcon_VerifImages($hImages)
	Local $Indice = -1
	For $i = 0 To UBound($AddIcon_ImageList, 1)-1
		If($AddIcon_ImageList[$i][0] = $hImages) Then
			$Indice = $i
			ExitLoop
		EndIf
	Next
	Return $Indice
EndFunc

Func __AddIcon_Exist($Indice, $FileName, $Index)
	Local $TabTemp = $AddIcon_ImageList[$Indice][4]
	Local $Exist = -1
	For $i = 0 To UBound($TabTemp, 1)-1
		If($TabTemp[$i][0] = $FileName And $TabTemp[$i][1] = $Index) Then
			$Exist = $TabTemp[$i][2]
			ExitLoop
		EndIf
	Next
	Return $Exist
EndFunc

Func __AddIcon_Add($hImages, $Indice, $File, $Index)
	Local $TabTemp = $AddIcon_ImageList[$Indice][4]
	Local $IndiceUbound  = UBound($TabTemp, 1)-1
	$TabTemp[$IndiceUbound][0] = $File
	$TabTemp[$IndiceUbound][1] = $Index
	$TabTemp[$IndiceUbound][2] = _GUIImageList_AddIcon($hImages, $File, $Index)
	If($TabTemp[$IndiceUbound][2] = -1) Then
		$TabTemp[$IndiceUbound][2] = $AddIcon_ImageList[$Indice][2]
	EndIf
	ReDim $TabTemp[UBound($TabTemp, 1)+1][UBound($TabTemp, 2)]
	$AddIcon_ImageList[$Indice][4] = $TabTemp
	Return $TabTemp[$IndiceUbound][2]
EndFunc

Func _AddIcon_GetDefaultDossier($hImages)
	Return __AddIcon_GetDefault($hImages, $AddIcon_DOSSIER)
EndFunc

Func _AddIcon_GetDefaultExe($hImages)
	Return __AddIcon_GetDefault($hImages, $AddIcon_EXE)
EndFunc

Func _AddIcon_GetDefaultInconnu($hImages)
	Return __AddIcon_GetDefault($hImages, $AddIcon_INCONNU)
EndFunc

Func __AddIcon_GetDefault($hImages, $Item)
 	Local $Retour = -1
	Local $Indice = __AddIcon_VerifImages($hImages)
	If($Indice >= 0 And $Item >= 1 And $Item <= 3) Then
		$Retour = $AddIcon_ImageList[$Indice][$Item]
	EndIf
	Return $Retour
EndFunc
