;~ This file is part of ClipKeeper.

;~     ClipKeeper is free software: you can redistribute it and/or modify
;~     it under the terms of the GNU General Public License as published by
;~     the Free Software Foundation, either version 3 of the License, or
;~     (at your option) any later version.

;~     ClipKeeper is distributed in the hope that it will be useful,
;~     but WITHOUT ANY WARRANTY; without even the implied warranty of
;~     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;~     GNU General Public License for more details.

;~     You should have received a copy of the GNU General Public License
;~     along with ClipKeeper.  If not, see <http://www.gnu.org/licenses/>.
#include <Constants.au3>
#include <WinAPI.au3>
#include <WinAPIRes.au3>

Global Const $CURSORS[15] = [$OCR_NORMAL, $OCR_IBEAM, $OCR_WAIT, $OCR_CROSS, $OCR_UP, $OCR_SIZE, $OCR_ICON, $OCR_SIZENWSE, _
							$OCR_SIZENESW, $OCR_SIZEWE, $OCR_SIZENS, $OCR_SIZEALL, $OCR_NO, $OCR_HAND, $OCR_APPSTARTING]
Global $Cursor_RollBack[UBound($CURSORS)]

For $i = 0 To UBound($CURSORS)-1
	$Cursor_RollBack[$i] = _CopyCursor(_LoadCursor($CURSORS[$i]))
Next
OnAutoItExitRegister("ExitCursor")

Func _ResetCursor($Cur)
	Local $Id = _GetIdByCur($Cur)
	If($Id >= 0 And $Cursor_RollBack[$Id] <> 0) Then
		_WinAPI_SetSystemCursor($Cursor_RollBack[$Id], $Cur, 1)
	EndIf
EndFunc

Func _ResetAllCursor()
	For $i = 0 To UBound($CURSORS)-1
		If($Cursor_RollBack[$i] <> 0) Then
			_WinAPI_SetSystemCursor($Cursor_RollBack[$i], $CURSORS[$i], 1)
		EndIf
	Next
EndFunc

Func _SetCursorFromIconFile($Cur, $File, $width = 16, $height = 16)
	Local $newCur = _WinAPI_LoadImage(0, $File, $IMAGE_ICON, $width, $height, $LR_LOADFROMFILE)
	_WinAPI_SetSystemCursor($newCur, $Cur, 1)
EndFunc

Func _SetAllCursorFromIconFile($File, $width = 16, $height = 16)
	For $i = 0 To UBound($CURSORS)-1
		Local $newCur = _WinAPI_LoadImage(0, $File, $IMAGE_ICON, $width, $height, $LR_LOADFROMFILE)
		_WinAPI_SetSystemCursor($newCur, $CURSORS[$i], 1)
	Next
EndFunc

Func _LoadCursorFromFile($iCursor)
    Return SetError(@error, @extended, _API(DllCall("user32.dll", "int", "LoadCursorFromFile", "str", $iCursor)))
EndFunc  ;==>_LoadCursorFromFile

Func _LoadCursor($iCursor)
    Return SetError(@error, @extended, _API(DllCall("user32.dll", "int", "LoadCursorA", "hwnd", 0, "int", $iCursor)))
EndFunc  ;==>_LoadCursor

Func _CopyCursor_($iCursor); <--- check your user32.dll for CopyCursor function. Is it there?
    Return SetError(@error, @extended, _API(DllCall("user32.dll", "int", "CopyCursor", "hwnd", $iCursor)))
EndFunc  ;==>_CopyCursor_

Func _DestroyCursor($iCursor)
    Return SetError(@error, @extended, _API(DllCall("user32.dll", "int", "DestroyCursor", "hwnd", $iCursor)))
EndFunc  ;==>_DestroyCursor

Func _API($v_ret)
    Local $err = @error
    Local $ext = @extended
    If Not $err Then
        If IsArray($v_ret) Then
            Return $v_ret[0]
        Else
            Return $v_ret
        EndIf
    EndIf
    Return SetError($err, $ext, 0)
EndFunc  ;==>_API


Func _CopyCursor($hCursor)

    Local $aCall = DllCall("user32.dll", "hwnd", "CopyImage", _
            "hwnd", $hCursor, _
            "dword", 2, _; IMAGE_CURSOR
            "int", 0, _; same width as the original
            "int", 0, _; same height as the original
            "dword", 16384); LR_COPYFROMRESOURCE

    If @error Or Not $aCall[0] Then
        Return SetError(1, 0, 0)
    EndIf

    Return SetError(0, 0, $aCall[0])

EndFunc  ;==>_CopyCursor

Func _GetIdByCur($Cur)
	Local $retour = -1
	For $i = 0 To UBound($CURSORS, 1)-1
		If($CURSORS[$i] = $Cur) Then
			$retour = $i
			ExitLoop
		EndIf
	Next
	Return $retour
EndFunc

Func ExitCursor()
	_ResetAllCursor()
EndFunc